﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;

namespace WeakEventListenerDemo
{
    public partial class Page : UserControl
    {
        ObservableCollection<int> _itemsSource;
        private WeakReference _leftReference;
        private WeakReference _rightReference;

        public Page()
        {
            InitializeComponent();

            // Create a class-level ObservableCollection
            _itemsSource = new ObservableCollection<int> { 1, 2, 3, 4, 5 };

            // Get a reference to both controls and hook up their ItemsSource properties
            var leakyControl = (LeakyControl)LeftPanel.Children[0];
            _leftReference = new WeakReference(leakyControl);
            leakyControl.ItemsSource = _itemsSource;
            var fixedControl = (FixedControl)RightPanel.Children[0];
            _rightReference = new WeakReference(fixedControl);
            fixedControl.ItemsSource = _itemsSource;
        }

        private void Status_Click(object sender, RoutedEventArgs e)
        {
            // Update the status for both controls
            LeftStatus.Text = "Status: " + (_leftReference.IsAlive ? "Present" : "Gone!");
            RightStatus.Text = "Status: " + (_rightReference.IsAlive ? "Present" : "Gone!");
        }

        private void Clear_Click(object sender, RoutedEventArgs e)
        {
            // Set the ItemsSource property of each control to null
            var leakyControl = _leftReference.Target as LeakyControl;
            if (null != leakyControl)
            {
                leakyControl.ItemsSource = null;
            }
            var fixedControl = _leftReference.Target as FixedControl;
            if (null != fixedControl)
            {
                fixedControl.ItemsSource = null;
            }
        }

        private void Remove_Click(object sender, RoutedEventArgs e)
        {
            // Remove both controls from the UI
            LeftPanel.Children.Clear();
            RightPanel.Children.Clear();
        }

        private void GC_Click(object sender, RoutedEventArgs e)
        {
            // Force a full garbage collection
            GC.Collect();
        }
    }
}
